#
# plugins.py - a simple plugin architecture for SNAKE inspired 
#              by yum's plugins.py
#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#

import glob,os,imp,sys

def _load_modules(path,slots):
    '''Generic module loader. Loads python modules from the given path (a list
    of directories). Finds each attribute of the module that matches one of the
    slots and adds it to a list of tuples of the form (modulename,attribute).
    The function returns a dict where the keys are the slots and the values
    are the lists of tuples.'''
    modules={}
    moduleitems={}
    for slot in slots:
        moduleitems[slot] = []
    for dir in path:
        if not os.path.isdir(dir): continue
        sys.path.insert(0,dir)
        for modulefile in glob.glob('%s/*.py' % dir):
            dir, modname = os.path.split(modulefile)
            modname = modname.replace(".py",'')
            if modname == "__init__":
                continue

            # attempt to load the module
            f,pathname,desc = imp.find_module(modname,[dir])
            try:
                module = imp.load_module(modname,f,pathname,desc)
            finally:
                if f: f.close()

            if not modules.has_key(modname):
                modules[modname] = module
            else:
                print "WARN: two modules named %s found" % modname
            for slot in slots:
                if hasattr(module,slot):
                    moduleitems[slot].append((modname,getattr(module,slot)))
        del sys.path[0]
    return moduleitems

SLOTS = ['kernel_args_hook','kickstart_hook','registration_hook']
def load_plugins(path):
    '''Return a dict - each key is a SLOT and each value is a list of
    (plugin_name,SLOT_function) tuples.'''
    return _load_modules(path,SLOTS)

def load_templates(path):
    '''Return a dict of templatename:Kickstart items'''
    templates={}
    for (name,func) in _load_modules(path,['ks'])['ks']:
        templates[name] = func
    return templates
